@@ -4,6 +4,8 @@ REST+Json protocol implementation.
 import datetime
 import decimal
 
+import six
+
 from simplegeneric import generic
 
 from wsme.protocols.rest import RestProtocol
@@ -44,21 +46,27 @@ def tojson(datatype, value):
     return value
 
 
-@tojson.when_type(list)
+@tojson.when_object(wsme.types.bytes)
+def bytes_tojson(datatype, value):
+    if value is None:
+        return None
+    return value.decode('ascii')
+
+
+@tojson.when_type(wsme.types.ArrayType)
 def array_tojson(datatype, value):
     if value is None:
         return None
-    return [tojson(datatype[0], item) for item in value]
+    return [tojson(datatype.item_type, item) for item in value]
 
 
-@tojson.when_type(dict)
+@tojson.when_type(wsme.types.DictType)
 def dict_tojson(datatype, value):
     if value is None:
         return None
-    key_type = datatype.keys()[0]
-    value_type = datatype.values()[0]
     return dict((
-        (tojson(key_type, item[0]), tojson(value_type, item[1]))
+        (tojson(datatype.key_type, item[0]),
+            tojson(datatype.value_type, item[1]))
         for item in value.items()
     ))
 
@@ -123,29 +131,28 @@ def fromjson(datatype, value):
     return value
 
 
-@fromjson.when_type(list)
+@fromjson.when_type(wsme.types.ArrayType)
 def array_fromjson(datatype, value):
     if value is None:
         return None
-    return [fromjson(datatype[0], item) for item in value]
+    return [fromjson(datatype.item_type, item) for item in value]
 
 
-@fromjson.when_type(dict)
+@fromjson.when_type(wsme.types.DictType)
 def dict_fromjson(datatype, value):
     if value is None:
         return None
-    key_type = datatype.keys()[0]
-    value_type = datatype.values()[0]
     return dict((
-        (fromjson(key_type, item[0]), fromjson(value_type, item[1]))
+        (fromjson(datatype.key_type, item[0]),
+            fromjson(datatype.value_type, item[1]))
         for item in value.items()))
 
 
-@fromjson.when_object(str)
+@fromjson.when_object(six.binary_type)
 def str_fromjson(datatype, value):
     if value is None:
         return None
-    return str(value)
+    return value.encode('utf8')
 
 
 @fromjson.when_object(decimal.Decimal)
@@ -212,10 +219,10 @@ class RestJsonProtocol(RestProtocol):
         r = tojson(context.funcdef.return_type, result)
         if self.nest_result:
             r = {'result': r}
-        return json.dumps(r, ensure_ascii=False).encode('utf8')
+        return json.dumps(r)
 
     def encode_error(self, context, errordetail):
-        return json.dumps(errordetail, encoding='utf-8')
+        return json.dumps(errordetail)
 
     def encode_sample_value(self, datatype, value, format=False):
         r = tojson(datatype, value)
