@@ -1,9 +1,9 @@
 import datetime
 
-try:
-    import xml.etree.ElementTree as et
-except ImportError:
-    import cElementTree as et
+from six import u
+import six
+
+import xml.etree.ElementTree as et
 
 from simplegeneric import generic
 
@@ -58,10 +58,12 @@ def toxml(datatype, key, value):
                             key, datatype.tobasetype(value))
         elif wsme.types.iscomplex(datatype):
             for attrdef in datatype._wsme_attributes:
-                el.append(toxml(attrdef.datatype, attrdef.name,
-                                getattr(value, attrdef.key)))
+                attrvalue = getattr(value, attrdef.key)
+                if attrvalue is not wsme.types.Unset:
+                    el.append(toxml(attrdef.datatype, attrdef.name,
+                                    attrvalue))
         else:
-            el.text = unicode(value)
+            el.text = six.text_type(value)
     return el
 
 
@@ -96,31 +98,31 @@ def fromxml(datatype, element):
             if sub is not None:
                 setattr(obj, attrdef.key, fromxml(attrdef.datatype, sub))
         return obj
+    if datatype is wsme.types.bytes:
+        return element.text.encode('ascii')
     return datatype(element.text)
 
 
-@toxml.when_type(list)
+@toxml.when_type(wsme.types.ArrayType)
 def array_toxml(datatype, key, value):
     el = et.Element(key)
     if value is None:
         el.set('nil', 'true')
     else:
         for item in value:
-            el.append(toxml(datatype[0], 'item', item))
+            el.append(toxml(datatype.item_type, 'item', item))
     return el
 
 
-@toxml.when_type(dict)
+@toxml.when_type(wsme.types.DictType)
 def dict_toxml(datatype, key, value):
     el = et.Element(key)
     if value is None:
         el.set('nil', 'true')
     else:
-        key_type = datatype.keys()[0]
-        value_type = datatype.values()[0]
         for item in value.items():
-            key = toxml(key_type, 'key', item[0])
-            value = toxml(value_type, 'value', item[1])
+            key = toxml(datatype.key_type, 'key', item[0])
+            value = toxml(datatype.value_type, 'value', item[1])
             node = et.Element('item')
             node.append(key)
             node.append(value)
@@ -128,6 +130,16 @@ def dict_toxml(datatype, key, value):
     return el
 
 
+@toxml.when_object(wsme.types.bytes)
+def bytes_toxml(datatype, key, value):
+    el = et.Element(key)
+    if value is None:
+        el.set('nil', 'true')
+    else:
+        el.text = value.decode('ascii')
+    return el
+
+
 @toxml.when_object(bool)
 def bool_toxml(datatype, key, value):
     el = et.Element(key)
@@ -158,22 +170,23 @@ def datetime_toxml(datatype, key, value):
     return el
 
 
-@fromxml.when_type(list)
+@fromxml.when_type(wsme.types.ArrayType)
 def array_fromxml(datatype, element):
     if element.get('nil') == 'true':
         return None
-    return [fromxml(datatype[0], item) for item in element.findall('item')]
+    return [
+        fromxml(datatype.item_type, item)
+        for item in element.findall('item')
+    ]
 
 
-@fromxml.when_type(dict)
+@fromxml.when_type(wsme.types.DictType)
 def dict_fromxml(datatype, element):
     if element.get('nil') == 'true':
         return None
-    key_type = datatype.keys()[0]
-    value_type = datatype.values()[0]
     return dict((
-        (fromxml(key_type, item.find('key')),
-            fromxml(value_type, item.find('value')))
+        (fromxml(datatype.key_type, item.find('key')),
+            fromxml(datatype.value_type, item.find('value')))
         for item in element.findall('item')))
 
 
@@ -220,7 +233,7 @@ class RestXmlProtocol(RestProtocol):
         return fromxml(arg.datatype, value)
 
     def parse_arg(self, name, value):
-        return et.fromstring(u"<%s>%s</%s>" % (name, value, name))
+        return et.fromstring(u("<%s>%s</%s>") % (name, value, name))
 
     def parse_args(self, body):
         return dict((sub.tag, sub) for sub in et.fromstring(body))
