@@ -3,15 +3,20 @@ import sys
 import traceback
 import weakref
 
+from six import u, b
+import six
+
 import webob
 
 from wsme import exc
 from wsme.protocols import getprotocol
 from wsme.api import scan_api
+from wsme import spore
+import wsme.types
 
 log = logging.getLogger(__name__)
 
-html_body = """
+html_body = u("""
 <html>
 <head>
   <style type='text/css'>
@@ -22,7 +27,32 @@ html_body = """
 %(content)s
 </body>
 </html>
-"""
+""")
+
+
+def default_prepare_response_body(request, results):
+    r = None
+    sep = None
+    for value in results:
+        if sep is None:
+            if isinstance(value, six.text_type):
+                sep = u('\n')
+                r = u('')
+            else:
+                sep = b('\n')
+                r = b('')
+        else:
+            r += sep
+        r += value
+    return r
+
+
+class DummyTransaction:
+    def commit(self):
+        pass
+
+    def abort(self):
+        pass
 
 
 class WSRoot(object):
@@ -31,16 +61,49 @@ class WSRoot(object):
 
     :param protocols: A list of protocols to enable (see :meth:`addprotocol`)
     :param webpath: The web path where the webservice is published.
+
+    :type  transaction: A `transaction
+                        <http://pypi.python.org/pypi/transaction>`_-like
+                        object or ``True``.
+    :param transaction: If specified, a transaction will be created and
+                        handled on a per-call base.
+
+                        This option *can* be enabled along with `repoze.tm2
+                        <http://pypi.python.org/pypi/repoze.tm2>`_
+                        (it will only make it void).
+
+                        If ``True``, the default :mod:`transaction`
+                        module will be imported and used.
+
     """
-    def __init__(self, protocols=[], webpath=''):
+    __registry__ = wsme.types.registry
+
+    def __init__(self, protocols=[], webpath='', transaction=None):
         self._debug = True
         self._webpath = webpath
         self.protocols = []
+
+        self._transaction = transaction
+        if self._transaction is True:
+            import transaction
+            self._transaction = transaction
+
         for protocol in protocols:
             self.addprotocol(protocol)
 
         self._api = None
 
+    def wsgiapp(self):
+        """Returns a wsgi application"""
+        from webob.dec import wsgify
+        return wsgify(self._handle_request)
+
+    def begin(self):
+        if self._transaction:
+            return self._transaction.begin()
+        else:
+            return DummyTransaction()
+
     def addprotocol(self, protocol, **options):
         """
         Enable a new protocol on the controller.
@@ -61,6 +124,8 @@ class WSRoot(object):
         """
         if self._api is None:
             self._api = [i for i in scan_api(self)]
+            for path, fdef in self._api:
+                fdef.resolve_types(self.__registry__)
         return self._api
 
     def _get_protocol(self, name):
@@ -70,13 +135,20 @@ class WSRoot(object):
 
     def _select_protocol(self, request):
         log.debug("Selecting a protocol for the following request :\n"
-                  "headers: %s\nbody: %s", request.headers,
-                  len(request.body) > 512
-                  and request.body[:512]
-                  or request.body)
+                  "headers: %s\nbody: %s", request.headers.items(),
+                  request.content_length and (
+                      request.content_length > 512
+                      and request.body[:512]
+                      or request.body)
+                  or '')
         protocol = None
+        path = str(request.path)
+        assert path.startswith(self._webpath)
+        path = path[len(self._webpath) + 1:]
         if 'wsmeproto' in request.params:
             return self._get_protocol(request.params['wsmeproto'])
+        elif path.startswith('_protocol'):
+            return self._get_protocol(path.split('/')[1])
         else:
 
             for p in self.protocols:
@@ -93,9 +165,9 @@ class WSRoot(object):
                 context.path = protocol.extract_path(context)
 
             if context.path is None:
-                raise exc.ClientSideError(
-                    u'The %s protocol was unable to extract a function '
-                    u'path from the request' % protocol.name)
+                raise exc.ClientSideError(u(
+                    'The %s protocol was unable to extract a function '
+                    'path from the request') % protocol.name)
 
             context.func, context.funcdef = self._lookup_function(context.path)
             kw = protocol.read_arguments(context)
@@ -104,7 +176,13 @@ class WSRoot(object):
                 if arg.mandatory and arg.name not in kw:
                     raise exc.MissingArgument(arg.name)
 
-            result = context.func(**kw)
+            txn = self.begin()
+            try:
+                result = context.func(**kw)
+                txn.commit()
+            except:
+                txn.abort()
+                raise
 
             if context.funcdef.protocol_specific \
                     and context.funcdef.return_type is None:
@@ -113,7 +191,8 @@ class WSRoot(object):
                 # TODO make sure result type == a._wsme_definition.return_type
                 return protocol.encode_result(context, result)
 
-        except Exception, e:
+        except Exception:
+            e = sys.exc_info()[1]
             infos = self._format_exception(sys.exc_info())
             if isinstance(e, exc.ClientSideError):
                 request.client_errorcount += 1
@@ -122,29 +201,42 @@ class WSRoot(object):
             return protocol.encode_error(context, infos)
 
     def _handle_request(self, request):
-        def default_prepare_response_body(request, results):
-            return '\n'.join(results)
-
         res = webob.Response()
         res_content_type = None
+
+        if request.path == self._webpath + '/api.spore':
+            res.body = spore.getdesc(self, request.host_url)
+            res.content_type = 'application/json'
+            return res
+
         try:
+            msg = None
             protocol = self._select_protocol(request)
-            if protocol is None:
+        except Exception:
+            e = sys.exc_info()[1]
+            msg = ("Error while selecting protocol: %s" % str(e))
+            log.exception(msg)
+            protocol = None
+
+        if protocol is None:
+            if msg is None:
                 msg = ("None of the following protocols can handle this "
                        "request : %s" % ','.join(
                             (p.name for p in self.protocols)))
-                res.status = 500
-                res.content_type = 'text/plain'
-                res.body = msg
-                log.error(msg)
-                return res
+            res.status = 500
+            res.content_type = 'text/plain'
+            res.text = u(msg)
+            log.error(msg)
+            return res
+
+        request.calls = []
+        request.client_errorcount = 0
+        request.server_errorcount = 0
+
+        try:
 
             context = None
 
-            request.calls = []
-            request.client_errorcount = 0
-            request.server_errorcount = 0
-
             if hasattr(protocol, 'prepare_response_body'):
                 prepare_response_body = protocol.prepare_response_body
             else:
@@ -153,8 +245,9 @@ class WSRoot(object):
             body = prepare_response_body(request, (
                 self._do_call(protocol, context)
                 for context in protocol.iter_calls(request)))
-            if isinstance(body, unicode):
-                res.unicode_body = body
+
+            if isinstance(body, six.text_type):
+                res.text = body
             else:
                 res.body = body
 
@@ -173,15 +266,14 @@ class WSRoot(object):
             else:
                 res.status = protocol.get_response_status(request)
                 res_content_type = protocol.get_response_contenttype(request)
-        except Exception, e:
+        except Exception:
             infos = self._format_exception(sys.exc_info())
             request.server_errorcount += 1
-            res.body = protocol.encode_error(context, infos)
+            res.text = protocol.encode_error(context, infos)
             res.status = 500
 
         if res_content_type is None:
             # Attempt to correctly guess what content-type we should return.
-            last_q = 0
             ctypes = [ct for ct in protocol.content_types if ct]
             if ctypes:
                 res_content_type = request.accept.best_match(ctypes)
@@ -190,7 +282,7 @@ class WSRoot(object):
         # output format.
         if res_content_type is None:
             if "text/html" in request.accept:
-                res.body = self._html_format(res.body, protocol.content_types)
+                res.text = self._html_format(res.body, protocol.content_types)
                 res_content_type = "text/html"
 
         # TODO should we consider the encoding asked by
@@ -263,11 +355,13 @@ class WSRoot(object):
             return html_body % dict(
                 css=formatter.get_style_defs(),
                 content=highlight(content, lexer, formatter).encode('utf8'))
-        except Exception, e:
+        except Exception:
+            e = sys.exc_info()[1]
             log.warning(
                 "Could not pygment the content because of the following "
                 "error :\n%s" % e)
             return html_body % dict(
                 css='',
-                content='<pre>%s</pre>' %
-                    content.replace('>', '&gt;').replace('<', '&lt;'))
+                content=u('<pre>%s</pre>') %
+                    content.replace(b('>'), b('&gt;'))
+                           .replace(b('<'), b('&lt;')))
